# srcds-commtest

This is a web-based tool to check for connectivity to Source game servers. It checks if a game server at the user's IP, and a user-supplied port number, is reachable. It checks for both TCP and UDP connectivity and informs the user of the result.

## Setup

This was created and tested using PHP 8.1. PHP modules required are TODO.

It also requires a MySQL server, preferably running on the same system that runs this application. This is because it uses the MySQL database for request counting and ratelimiting.

1. Install and configure MySQL or MariaDB
1. Install a Web server (i.e. Apache) and PHP
1. Place the files from this repository somewhere. Make the 'www' subfolder of the repository your website root.
1. Configure the settings in `settings.php`
1. Create a cron job to execute `database-cleanup.php` every hour

```
0 *   *   *   *    php -f /home/tracker/commtest/database-cleanup.php
```

That's basically it.

## Use

There are two ways to use it, like an API (where it produces simple text-based results) and as a webpage.

Webpage mode is intended to be run in an iframe embedded on the main GE:S website. If you don't have "src=iframe" as a query string, you will be redirected to the URL defined in `settings.php`.

API mode is triggered if the HTTP URI begins with "/api". I.e. if the user is accessing the site at the URL `servertest.geshl2.com/api`, API mode is automatically triggered. This depends on rewrite rules similar to the below (the below example was written for Apache):

```
RewriteEngine  on
RewriteRule    "^/api[/]{0,1}$"  "/" [PT]
```

### API mode documentation

To call the API, simply call the URL and append `?port=<port_number>` to the URL, such as `https://servertest.geshl2.com/api/?port=27015`.

The `port` query string is required in order to run the check (this is to prevent discovery by spambots, web crawlers, etc from triggering it).

When calling the API it will return one of the following strings:

* tcpudp
* tcponly
* udponly
* unreachable
* ratelimited (also sets the HTTP response code to 429)

If the API returns `ratelimited`, the requesting IP has used the maximum number of available requests for the current hour.

### Webpage mode documentation

Webpage mode is intended to be embedded on the GE:S website. On the server running the webpage, be sure to set the `Content-Security-Policy` header to prevent embedding from unauthorized domains!

It is also intended that you pass query strings from the website into the iframe, so that you can make use of the `port` and `runtest` query strings. The main website is currently generated with a static-site generator, so we accomplish this using javascript as of writing.

* `port`: specifies the port number to test
* `runtest`: does the connectivity test upon loading the page. Can be useful when loaded from elsewhere, like the connectivity test shortcuts generated by the server installer.