<?php

// These functions pertain to rate-limiting the service.

// Number of requests per hour until the requestor gets blocked
define("MAX_REQUESTS_HR", 180);

// this function handles the MySQL database connection
function db_conn()
{

  static $DB_CONN;

  // If we've already run once, return the results of last time
  if( !empty($DB_CONN) )
  {
    return($DB_CONN);
  }

  try {
    // Create a new, persistent connection.
    $DB_CONN = new PDO('mysql:host=' . MYSQL_SV . ';dbname='.MYSQL_DB, MYSQL_USER, MYSQL_PASS, array(
      PDO::ATTR_PERSISTENT => true));
    // set the PDO error mode to exception
    $DB_CONN->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
    // enable real prepared statements, preventing SQL injection
    $DB_CONN->setAttribute(PDO::ATTR_EMULATE_PREPARES, false);
    // Default fetch() to return associative arrays
    $DB_CONN->setAttribute(PDO::ATTR_DEFAULT_FETCH_MODE, PDO::FETCH_ASSOC);
  }
  catch(PDOException $e)
  {
    return FALSE;
  }

  // set time zone to UTC
  $DB_CONN->query("SET time_zone = '+00:00';");
  return $DB_CONN;
}

// Return the current timeslot, for 
function ratelimit_get_timeslot()
{

	$time = time(); // get unix time
	$time = $time - $time % 3600; // round down to hour
	$time = date('Y-m-d H:i:s', $time); // convert to MySQL datetime format
	return($time);

}

// Increment the request count for the requesting IP
function ratelimit_counter_add()
{

	// get the current timeslot (hour)
	$timeslot = ratelimit_get_timeslot();

	// add this request to the counter
	$sqlQuery = db_conn()->prepare('
		INSERT INTO REQUEST_COUNTERS (RTIME, IPv4)
		VALUES (:timeslot, INET_ATON(:ip))

		ON DUPLICATE KEY
		UPDATE RTIME = RTIME, COUNTER = COUNTER +1;
	');

	$sqlQuery->execute(array(
		'timeslot' => $timeslot,
		'ip' => $_SERVER['REMOTE_ADDR']
	));

	// Check if user exceeded the limit for this timeslot.
	$sqlQuery = db_conn()->prepare('
		SELECT COUNTER FROM REQUEST_COUNTERS
		WHERE IPv4 = INET_ATON(:ip)
		AND RTIME = :timeslot
	');

	$sqlQuery->execute(array(
		'timeslot'=>$timeslot,
		'ip'=>$_SERVER['REMOTE_ADDR']
	));

	$counter = $sqlQuery->fetch()['COUNTER'];

	// If the limit was exceeded, ban the requestor for the remainder of this timeslot.
	if( $counter > MAX_REQUESTS_HR )
	{

		$sqlQuery = db_conn()->prepare('
			INSERT IGNORE INTO BLOCKED_IPS (IPv4)
			VALUES (INET_ATON(:ip))
		');

		$sqlQuery->execute(array(
			'ip'=>$_SERVER['REMOTE_ADDR']
		));

	}

}

// Checks if the current requesting IP is ratelimited/blocked
function is_ratelimited()
{

	$sqlQuery = db_conn()->prepare('
		SELECT IPv4 FROM BLOCKED_IPS
		WHERE IPv4 = INET_ATON(:ip)
	');

	$sqlQuery->execute(array(
		'ip'=>$_SERVER['REMOTE_ADDR']
	));

	$results = $sqlQuery->rowCount();

	// If a non-zero number of rows are returned, the IP is banned
	if( $results != 0 )
	{

		return(TRUE);

	} else {

		return(FALSE);

	}

}