-- A small database used for rate limiting.

-- The RTIME is a timestamp rounded down to the hour, i.e. unixtime - (unixtime mod 3600)

-- IPv4 is the IPv4 address of the client, in integer form (INET_ATON)

-- The COUNTER will be the number of times during the current hour that the
-- IP requested a check.

-- The request counters are frequently updated, and as such, benefit from
-- being a MEMORY table.

CREATE TABLE REQUEST_COUNTERS
(
  RTIME TIMESTAMP NOT NULL,
  IPv4 INT UNSIGNED NOT NULL,
  COUNTER SMALLINT UNSIGNED NOT NULL DEFAULT 1,
  UNIQUE KEY (RTIME, IPv4)
) ENGINE=MEMORY;

CREATE TABLE BLOCKED_IPS
(
  BTIME TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
  IPv4 INT UNSIGNED NOT NULL,
  PRIMARY KEY (IPv4), KEY (BTIME)
);

DELIMITER //

-- Being a memory table, this trigger limits the number of request counters.
-- The table, at capacity, is small enough that it should not need an increased
-- max_heap_table size.

CREATE TRIGGER REQUEST_COUNTER_CAP BEFORE INSERT ON REQUEST_COUNTERS FOR EACH ROW
BEGIN

  IF (SELECT COUNT(*) FROM REQUEST_COUNTERS) >= 8192
  THEN
    SIGNAL SQLSTATE '45000'
      SET MESSAGE_TEXT = 'Request counter limit reached.';
  END IF;

END;

//

DELIMITER ;
